% File: condlogprob.m
%
% Coded by: Gustav Eje Henter
% 
% clps = condlogprob(model,cond,nextobs)
% Computes conditional probabilities
% 
% Inputs:
% model:  A single probabilistic model that implements the logprob function
% cond:   A sequence of observations defining the conditional context
% nextobs (N * T): N sequences of T future observations each
% 
% Output:
% clps: The conditional pdf P(x_future=nextobs | x_past=cond ;model)
%       evaluated for each of the N sequences in nextobs
% 
% Note: This implementation is not optimised for speed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function clps = condlogprob(model,cond,nextobs)

[nseq,nobs] = size(nextobs);

need_to_remove_lp_of_cond = true;

clps = zeros(nseq,1);
for i = 1:nseq
    seqlp = model.logprob([cond,nextobs(i,:)]);
    
    % There are two options here:
    % Some simple models compute the aggregate logprob; other models
    % (including KDE-MM/KDE-HMM) return the logprob for each observation
    if numel(seqlp) > 1
        % The above logic can go wrong if size(cond,2) == model order
        need_to_remove_lp_of_cond = false;
        clps(i) = sum(seqlp(end-nobs+1:end));
    else
        clps(i) = seqlp; % seqlp is scalar; assume it's for the whole seq.
    end
end

if need_to_remove_lp_of_cond
    clps = clps - model.logprob(cond);
end