% file: run_AR_HMM_Gamma.m 
%
% with test statistic T ~ N(0,1): equal pdf under H0
% 4 distribution functions, Newey - West correction for autocorrelation
%
% Coded by: JDG
%
% Table 2, Example 2
% Y_t = (2/3)Y_{t-1}+sigma(Q_t)\varepsilon_{t}
%
% generate time series based on input parameters
% Number of states (NS) = 3
% function = ar_hmm_gen(N,S,pi,A,V)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

datastr =' ...    ';
warning('off','all')

default_opts3;  % with maxorderm = 5, maxorderh = 3, maxstates = 3

ntrain = 1000;  % training  KEEP it FIXED
ntest  = 1000;  % validation KEEP it FIXED
nr     = ntrain+ntest;
low    = 1;     % low = step "s" number or replications CHANGE
high   = low;

hzon   = 1;               % forecast horizon, CHANGE  
hmax   = 2;               % maximum forecast horizon
window =  49;  % 100;     % (ntrain-ntest-hmax+1);
% 500-450-2+1=49 rolling "hzon=1 or 2" forecasts

AR = 2/3;
A  = [AR, AR, AR];     % AR coefficients
V  = [0.3, 0.4, 0.5];  % sigma(Q_t), standard deviation for driving noise
pi = [0.8, 0.1, 0.1; 0.1, 0.8, 0.1; 0.1, 0.1, 0.8]; % transition matrix

ntrain = 1000;  
ntest  = 1000;  
nr     = ntrain+ntest;

n      = 5500; % total sample size, 550 for training
NS     = 3;    % number of states
warmup = 2000;

stat   = zeros(1,n);
states = repelem(-1,n);
obs    = repelem(0,n);
TM     = zeros(NS,NS);
TM     = pi;
T      = warmup+n;

% generate time series   
for j = 1:n
   if (j == 1 & states(1) <0)
      [nr,nc] = size(TM);
      p_state = repelem(1.0/nr,nr);
      state   = mnrnd(1,p_state,1);
      [rows, cols] = find(state==1);
      stat(j) = cols;
   end
      
   if (j >= 2 )
      id = cols;   
      p_state = TM(id,:);
      state   = mnrnd(1,p_state,1);
      [rows, cols] = find(state==1); 
      stat(j) = cols;
    end

    % sample current observation by using AR(1) process
    s = stat(j);
    a = A(s);
    e = V(s);
      
    if j == 1
      mu = 0;  %  assume the previous observation is equal to 0
    else
      mu = a*obs(j-1);
    end
    
%     obs(j) = mu + e*normrnd(0,1);  % standard N(0,1)
%     obs(j) = mu + e*(0.5*normrnd(-3,1)+0.5*normrnd(3,1));  % bimodal mixture
      obs(j) = mu + e*gamrnd(2,2);   % Gamma(2,2)
%     obs(j) = mu + e*lognrnd(0,1);  % log-normal
%     obs(j) = mu + e*trnd(2);       % Student t with df = 2  
      
end % end loop j=1:n

%% yy = obs;
yy = zscore(obs);     % mean = 0, sd = 1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
crit  = 1.96;   % 5% critical value T ~ N(0,1), 2-sided
crit2 = 1.645;  % 5% one-sided

clps4_n = zeros(window,1);
clps3_n = zeros(window,1);
clps2_n = zeros(window,1);
clps1_n = zeros(window,1);

for uu = 1:window  % Start rolling forecasting loop

rng('shuffle')
  value = [low  uu];
  disp('step (low) and  uu = ')
  disp(value)
  ntrain = 1000;   
  
  xt = yy(uu+ntrain:(ntrain+uu+ntrain-1));    % training 
  xf = yy(ntrain+ntrain+uu+hzon-1);           % forecast values
  xv = yy(ntrain+ntrain+uu:(ntrain+ntrain+uu+nr-1));
  
  nr_xt(uu) = length(xt);
  nr_xv(uu) = length(xv);
           
  xmin   = min(xt)-10;
  xmax   = max(xt)+10;
  xpts   = linspace(xmin,xmax,ntrain);
  cond   = xt;   % condition on the 500 points of the set xs, rolling

  maxima = estimate_discrete_maxima(xt);  % obtain maxima
  maxima = maxima(xt(maxima) > 50);       % Extract non-noise peaks, CHANGE 50

  gammas = phaseinit(datastr,ntrain,maxstates,maxima);

 kdehmm_experiment6_M3(datastr,ntrain,ntest,gridres,condq,maxorderm,maxorderh,...
   maxstates,maxiter,nreinits,inititer,miniter,minstep,gammas,...
   hmmgammainit,xv,xt);    

% Compute conditional pdf at xpts and xf  
  load('matfile.mat')
  model1      = mmpts(maxorderm+1,1);         
  clps1_n(uu) = condlogprob(model1,cond,xf);      % stationary KDE-MM
  
  model2      = hmmt(end);          
  clps2_n(uu) = condlogprob(model2,cond,xf);      % HMM

  model3      = art(1,2);        
  clps3_n(uu) = condlogprob(model3,cond,xf);      % AR-HMM

  model4      = kdeht(1,1);         
  clps4_n(uu) = condlogprob(model4,cond,xf);      % KDE-HMM
    
  dif_12(uu) = clps1_n(uu) - clps2_n(uu); % KDE_MM vs G-HMM
  dif_13(uu) = clps1_n(uu) - clps3_n(uu); % KDE_MM vs AR-HMM
  dif_14(uu) = clps1_n(uu) - clps4_n(uu); % KDE_MM vs KDE-HMM
  dif_23(uu) = clps2_n(uu) - clps3_n(uu); % G-HMM  vs AR-HMM
  dif_24(uu) = clps2_n(uu) - clps4_n(uu); % G_HMM  vs KDE-HMM
  dif_34(uu) = clps3_n(uu) - clps4_n(uu); % AR_HMM vs KDE-HMM
  
  delta1(uu) = (xf - clps1_n(uu))^2;
  delta2(uu) = (xf - clps2_n(uu))^2;  
  delta3(uu) = (xf - clps3_n(uu))^2;
  delta4(uu) = (xf - clps4_n(uu))^2;  
  
  forecast(uu) = xf;  % forecast values

end  % end rolling forecasting loop uu=1:window

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clc; 
crit2 = 1.645;
crit  = 1.96;

m_12 = mean(dif_12); 
m_13 = mean(dif_13);
m_14 = mean(dif_14);
m_23 = mean(dif_23);
m_24 = mean(dif_24);
m_34 = mean(dif_34);

s_12 = std(dif_12); 
s_13 = std(dif_13);
s_14 = std(dif_14);
s_23 = std(dif_23);
s_24 = std(dif_24);
s_34 = std(dif_34);

%%%   Giacomini - White test %%%%%
T12 = sqrt(window) * m_12 / std(dif_12);
T13 = sqrt(window) * m_13 / std(dif_13);
T14 = sqrt(window) * m_14 / std(dif_14);
T23 = sqrt(window) * m_23 / std(dif_23);
T24 = sqrt(window) * m_24 / std(dif_24);
T34 = sqrt(window) * m_34 / std(dif_34);

%% calculate the frequency of rejecting H0: equal performance of pdfs
rej12 = 0;
rej13 = 0;
rej14 = 0;
rej23 = 0;
rej24 = 0;
rej34 = 0;

if (T12 <= -crit || T12 >= crit) % KDE_MM vs G-HMM, || = OR
  rej12 = rej12 + 1;
end

if T12<0;
    sign_T12 ='(-)';
elseif  T12 >0;
    sign_T12 ='(+)';
end;

if (T13<= -crit || T13 >= crit) % KDE_MM vs AR-HMM
  rej13 = rej13 + 1;
end

if T13<0;
    sign_T13 ='(-)';
elseif  T13 >0;
    sign_T13 ='(+)';
end;

if (T14 <= -crit || T14 >= crit) % KDE_MM vs KDE-HMM
  rej14 = rej14 + 1;
end

if T14<0;
    sign_T14 ='(-)';
elseif  T14 >0;
    sign_T14 ='(+)';
end;

if (T23 <= -crit || T23 >= crit) % G_MM vs AR-HMM
  rej23 = rej23 + 1;
end

if T23<0;
    sign_T23 ='(-)';
elseif  T23 >0;
    sign_T23 ='(+)';
end;

if (T24 <= -crit || T24 >= crit) % G_MM vs KDE-HMM
  rej24 = rej24 + 1;
end

if T24<0;
    sign_T24 ='(-)';
elseif  T24 >0;
    sign_T24 ='(+)';
end;

if (T34 <= -crit || T34 >= crit) % AR_HMM vs KDE-HMM
  rej34 = rej34 + 1;
end

if T34<0;
    sign_T34 ='(-)';
elseif  T34 >0;
    sign_T34 ='(+)';
end;

%%%%%%%%%%%%%%% crit2 = 1.645 %%%%%%%%%%%%%%%%%%
rej12_2 = 0;
rej13_2 = 0;
rej14_2 = 0;
rej23_2 = 0;
rej24_2 = 0;
rej34_2 = 0;

if (T12 <= -crit2 || T12 >= crit2) % KDE_MM vs G-HMM, || = OR
  rej12_2 = rej12_2 + 1;
end

if T12<0;
    sign_T12_2 ='(-)';
elseif  T12 >0;
    sign_T12_2 ='(+)';
end;

if (T13 <= -crit2 || T13 >= crit2) % KDE_MM vs AR-HMM
  rej13_2 = rej13_2 + 1;
end

if T13<0;
    sign_T13_2 ='(-)';
elseif  T13 >0;
    sign_T13_2 ='(+)';
end;

if (T14 <= -crit2 || T14 >= crit2) % KDE_MM vs KDE-HMM
  rej14_2 = rej14_2 + 1;
end

if T14<0;
    sign_T14_2 ='(-)';
elseif  T14 >0;
    sign_T14_2 ='(+)';
end;

if (T23 <= -crit2 || T23 >= crit2) % G_MM vs AR-HMM
  rej23_2 = rej23_2 + 1;
end

if T23<0;
    sign_T23_2 ='(-)';
elseif  T23 >0;
    sign_T23_2 ='(+)';
end;

if (T24 <= -crit2 || T24 >= crit2) % G_MM vs KDE-HMM
  rej24_2 = rej24_2 + 1;
end

if T24<0;
    sign_T24_2 ='(-)';
elseif  T24 >0;
    sign_T24_2 ='(+)';
end;

if (T34 <= -crit2 || T34 >= crit2) % AR_HMM vs KDE-HMM
  rej34_2 = rej34_2 + 1;
end

if T34<0;
    sign_T34_2 ='(-)';
elseif  T34 >0;
    sign_T34_2 ='(+)';
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% CPA test statistic value (Giacomini - White)  Wald-type test
% reject H0: E( Loss(model A) - Loss(model B) )= 0. 
% The sign of the test-statistic
% indicates which forecast performs better:
% a + (positive) test-statistic indicates
% that model A forecast produces larger average loss than 
% the model B forecast 
% (model B outperforms model A), while 
% a - (negative) sign indicates the opposite.

[CPA_pv_12 sign12] = CPAtest2(delta1', delta2'); % KDE_MM vs G-HMM 
[CPA_pv_13 sign13] = CPAtest2(delta1', delta3'); % KDE_MM vs AR-HMM  (table) 
[CPA_pv_14 sign14] = CPAtest2(delta1', delta4'); % KDE_MM vs KDE_HMM (table)
[CPA_pv_23 sign23] = CPAtest2(delta2', delta3'); % G-HMM  vs AR-HMM
[CPA_pv_24 sign24] = CPAtest2(delta2', delta4'); % G-HMM  vs KDE-HMM 
[CPA_pv_34 sign34] = CPAtest2(delta3', delta4'); % AR-HMM vs KDE-HMM (table)
   
%%%%%%%%%%%%%%%%%% OUTPUT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp(' %%%%%%%%%%%%%%%%%% 1.96 %%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('crit = 1.96 rej12 rej 13 rej14 rej3 rej24 rej 34 ')
rejection =[rej12 rej13 rej14 rej23 rej24 rej34];
disp(rejection)

sign_AW = [sign_T12 sign_T13 sign_T14 sign_T23 sign_T24 sign_T34];
disp('sign-Amisano-Giacomini')
disp(sign_AW)

disp(' ')
disp(' %%%%%%%%%%%%%%%%%% 1.645 %%%%%%%%%%%%%%%%%%%%%%%%% ')

disp(' crit2 = 1.645 rej12_2 rej 13_2 rej14_2 rej23_2 rej24_2 rej34_2 ')
rejection_2 =[rej12_2 rej13_2 rej14_2 rej23_2 rej24_2 rej34_2];
disp(rejection_2)

sign_AW_2 = [sign_T12_2 sign_T13_2 sign_T14_2 sign_T23_2 sign_T24_2 sign_T34_2];
disp('crit2 = 1.645 sign-Amisano-Giacomini')
disp(sign_AW_2)

disp(' ')
disp(' %%%%%%%%%%%%%%%%%% CPA %%%%%%%%%%%%%%%%%%%%%%%%% ')

CPA=[CPA_pv_12 CPA_pv_13 CPA_pv_14 CPA_pv_23 CPA_pv_24 CPA_pv_34];
disp('p-values: CPA12 CPA13 CPA14 CPA23 CPA24 CPA34 ')
disp(CPA)

sign_GW = [sign12 sign13 sign14 sign23 sign24 sign34];
disp('sign-Giacomini-White')
disp(sign_GW)

%%%%%%%%%%% Original %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
prob1 = exp(clps1_n);
prob2 = exp(clps2_n);
prob3 = exp(clps3_n);
prob4 = exp(clps4_n);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%  Newey - West   %%%%%%%%%%

% Calculate the variance of the loss differential, taking into account
% autocorrelation.

gamma0 = var(dif_12);
d = dif_12';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP; 
    end
    var_12 = gamma0 + 2*sum(gamma);
else
    var_12 = gamma0;
end

gamma0 = var(dif_13);
d = dif_13';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP; 
    end
    var_13 = gamma0 + 2*sum(gamma);
else
    var_13 = gamma0;
end

gamma0 = var(dif_14);
d = dif_14';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP; 
    end
    var_14 = gamma0 + 2*sum(gamma);
else
    var_14 = gamma0;
end

gamma0 = var(dif_14);
d = dif_14';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP; 
    end
    var_14 = gamma0 + 2*sum(gamma);
else
    var_14 = gamma0;
end

gamma0 = var(dif_23);
d = dif_23';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP; 
    end
    var_23 = gamma0 + 2*sum(gamma);
else
    var_23 = gamma0;
end

gamma0 = var(dif_24);
d = dif_24';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP;
    end
    var_24 = gamma0 + 2*sum(gamma);
else
    var_24 = gamma0;
end

gamma0 = var(dif_34);
d = dif_34';
PP = window;
if hzon > 1  % number of forecasts
    gamma = zeros(hzon-1,1);
    for i = 1:hzon-1
        gamma(i) = ( d(1+i:PP)' * d(1:PP-i) ) ./ PP; 
    end
    var_34 = gamma0 + 2*sum(gamma);
else
    var_34 = gamma0;
end

%%%  Giacomini-White test  %%%%%
DT12 = sqrt(window) * m_12 / sqrt(var_12);
DT13 = sqrt(window) * m_13 / sqrt(var_13);
DT14 = sqrt(window) * m_14 / sqrt(var_14);
DT23 = sqrt(window) * m_23 / sqrt(var_23);
DT24 = sqrt(window) * m_24 / sqrt(var_24);
DT34 = sqrt(window) * m_34 / sqrt(var_34);

%%%%%%%%%%%%%%% crit2 = 1.645 %%%%%%%%%%%%%%%%%%
Drej12 = 0;
Drej13 = 0;
Drej14 = 0;
Drej23 = 0;
Drej24 = 0;
Drej34 = 0;

if (DT12 <= -crit2 || DT12 >= crit2) % KDE_MM vs G-HMM, || = OR
  Drej12 = Drej12 + 1;
end

if DT12<0;
    sign_DT12 ='(-)';
elseif  DT12 >0;
    sign_DT12 ='(+)';
end;

if (DT13 <= -crit2 || DT13 >= crit2) % KDE_MM vs AR-HMM
  Drej13 = Drej13 + 1;
end

if DT13<0;
    sign_DT13 ='(-)';
elseif  DT13 >0;
    sign_DT13 ='(+)';
end;

if (DT14 <= -crit2 || DT14 >= crit2) % KDE_MM vs KDE-HMM
  Drej14 = Drej14 + 1;
end

if DT14<0;
    sign_DT14 ='(-)';
elseif  DT14 >0;
    sign_DT14 ='(+)';
end;

if (DT23 <= -crit2 || DT23 >= crit2) % G_MM vs AR-HMM
  Drej23 = Drej23 + 1;
end

if DT23<0;
    sign_DT23 ='(-)';
elseif  DT23 >0;
    sign_DT23 ='(+)';
end;

if (DT24 <= -crit2 || DT24 >= crit2) % G_MM vs KDE-HMM
  Drej24 = Drej24 + 1;
end

if DT24<0;
    sign_DT24 ='(-)';
elseif  DT24 >0;
    sign_DT24 ='(+)';
end;

if (DT34 <= -crit2 || DT34 >= crit2) % AR_HMM vs KDE-HMM
  Drej34 = Drej34 + 1;
end

if DT34<0;
    sign_DT34 ='(-)';
elseif  DT34 >0;
    sign_DT34 ='(+)';
end;

disp(' ')
disp(' %%%%%%%%%%%%%%%%%% 1.645 Newey-West %%%%%%%%%%%%%%%%%%%%%%%%% ')

disp(' crit2 = 1.645 Drej12 Drej13 Drej14 Drej23 Drej24 Drej34 ')
rejection_3 =[Drej12 Drej13 Drej14 Drej23 Drej24 Drej34];
disp(rejection_3)

sign_AW_3 = [sign_DT12 sign_DT13 sign_DT14 sign_DT23 sign_DT24 sign_DT34];
disp('crit2 = 1.645 sign  Newey-West')
disp(sign_AW_3)

%%%%%%%%%%%%%%%%%%%%% END MAIN  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

